Exception: Clacky::Channel::Adapters::Feishu::FeishuDocPermissionError
- Inherits:
-
StandardError
- Object
- StandardError
- Clacky::Channel::Adapters::Feishu::FeishuDocPermissionError
- Defined in:
- lib/clacky/server/channel/adapters/feishu/bot.rb
Overview
Raised when the app lacks read permission for a specific Feishu document (error code 91403). The user needs to add the app as a collaborator on the document.
Instance Attribute Summary collapse
-
#doc_token ⇒ Object
readonly
Returns the value of attribute doc_token.
Instance Method Summary collapse
-
#initialize(doc_token) ⇒ FeishuDocPermissionError
constructor
A new instance of FeishuDocPermissionError.
Constructor Details
#initialize(doc_token) ⇒ FeishuDocPermissionError
Returns a new instance of FeishuDocPermissionError.
16 17 18 19 |
# File 'lib/clacky/server/channel/adapters/feishu/bot.rb', line 16 def initialize(doc_token) @doc_token = doc_token super("App has no permission to access document: #{doc_token}") end |
Instance Attribute Details
#doc_token ⇒ Object (readonly)
Returns the value of attribute doc_token.
14 15 16 |
# File 'lib/clacky/server/channel/adapters/feishu/bot.rb', line 14 def doc_token @doc_token end |