Class: Clacky::Tools::Edit
Instance Method Summary collapse
- #execute(path:, old_string:, new_string:, replace_all: false, working_dir: nil) ⇒ Object
- #format_call(args) ⇒ Object
- #format_result(result) ⇒ Object
Methods inherited from Base
#category, #description, #name, #parameters, #to_function_definition
Instance Method Details
#execute(path:, old_string:, new_string:, replace_all: false, working_dir: nil) ⇒ Object
34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 72 73 74 75 76 77 78 79 80 81 82 83 84 85 |
# File 'lib/clacky/tools/edit.rb', line 34 def execute(path:, old_string:, new_string:, replace_all: false, working_dir: nil) # Expand ~ to home directory, resolve relative paths against working_dir path = (path, working_dir: working_dir) unless File.exist?(path) return { error: "File not found: #{path}" } end unless File.file?(path) return { error: "Path is not a file: #{path}" } end begin # Scrub invalid UTF-8 bytes at read time — otherwise editing a file # that contains non-UTF-8 bytes would poison history / error messages # and cause JSON.generate to fail during replay. content = safe_utf8(File.read(path)) # Find matching string using layered strategy (shared with preview) match_result = Utils::StringMatcher.find_match(content, old_string) unless match_result return build_helpful_error(content, old_string, path) end actual_old_string = match_result[:matched_string] occurrences = match_result[:occurrences] # If not replace_all and multiple occurrences, warn about ambiguity if !replace_all && occurrences > 1 return { error: "String appears #{occurrences} times in the file. Use replace_all: true to replace all occurrences, " \ "or provide a more specific string that appears only once." } end # Perform replacement content = replace_all ? content.gsub(actual_old_string, new_string) : content.sub(actual_old_string, new_string) File.write(path, content) { path: File.(path), replacements: replace_all ? occurrences : 1, error: nil } rescue Errno::EACCES => e { error: "Permission denied: #{e.}" } rescue StandardError => e { error: "Failed to edit file: #{e.}" } end end |
#format_call(args) ⇒ Object
122 123 124 125 |
# File 'lib/clacky/tools/edit.rb', line 122 def format_call(args) path = args[:file_path] || args["file_path"] || args[:path] || args["path"] "Edit(#{Utils::PathHelper.safe_basename(path)})" end |
#format_result(result) ⇒ Object
127 128 129 130 131 132 |
# File 'lib/clacky/tools/edit.rb', line 127 def format_result(result) return result[:error] if result[:error] replacements = result[:replacements] || result["replacements"] || 1 "Modified #{replacements} occurrence#{replacements > 1 ? "s" : ""}" end |